 # -*- coding: utf-8 -*-
"""
Created on Mon Feb 18 23:27:35 2019

@author: Oriol Solé

Programa versió1: Enviament d'imatges automàtic (Programa modem transmissor)
"""

#Llibreries####################################################################

#Importacio de les llibreries adients:
import base64   #Llibreria per codificar i descodificar les imatges en base 64
import socket   #Llibreria per fer ús dels sockets
import time     #Llibreria per manipular tasques relacionades amb temps
import datetime #Llibreria per obentir dates
import os       #Llibreria per manipular paths
import numpy    #Libreria per executar operacions estadístiques
import xlsxwriter #Llibreria per operar amb fitxers excel 
from picamera import PiCamera#Llibreria per controlar la camera de la Raspberry

#Definicio Modems##############################################################

#Definicio de les IPs i dels ports de comunicació dels modems acustics físics 
#i virtuals transmissors
HOSTNAME1EM = '10.42.57.2'
HOSTNAME1 = '192.168.1.196'
PORT1 = 9200

#Classe funcions sockets#######################################################

#Creacio de la classe netcat per ubicar les funcions relatives a la manipulacio
#dels sockets
class netcat(object):
    #Definició de la funció inicial per la configuració del mòdem acústic
    def __init__ (self,hostname,port,modem_name):
        #Temps d'espera per assegurar la connectivitat ethernet de la Raspberry
        #al iniciar-se 
        time.sleep(10)
        #Establiment de la ip, el port de comunicació i el nom del mòdem
        self.ip = hostname
        self.port = port
        self.name = modem_name
        #Creació del socket i connectivitat
        try:
            self.netcat = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.netcat.settimeout(0.1)     
            self.netcat.connect((self.ip, self.port))
        except:
            print "No s'ha pogut establir connexio"
        
        #Configuracions inicials:
        print 'Activacio permissos'
        self.send('AT@CTRL')
        print 'Desactivacio notificacions extesses'
        self.send('AT@ZX0')
        print 'Desactivacio notificacions de posicionament'
        self.send('AT@ZU0')
        print 'Reiniciament del buffer de transmissio:'
        self.send('ATZ4')
        print 'Configuracio del numero de bytes del buffer de transmissio:'
        self.send('AT@ZL1000000')
        print 'Guardar en memoria les configuracions del modem'
        self.send('AT&W') 
    
    #Funció de lectura de dades 
    def port_read(self):
        buff = ''
        read = ''
        while 1:
            #Bloc d'excepcio, en cas de no poder establir connectivitat, i que
            #no es tracti d'un error de timeout, es torna a intentar restablir
            #la connexio perduda
            try:
                read = self.netcat.recv(1)
            except socket.error as error:
                if str(error)!= "timed out":
                    try:
                        self.netcat = socket.socket(socket.AF_INET,
                        socket.SOCK_STREAM)
                        self.netcat.settimeout(0.1)     
                        self.netcat.connect((self.ip, self.port))
                        print "Connexio recuperada"
                    except:
                        print "Intent de connexio fallit"
                        time.sleep(2)
                        pass
                break
            if read:
              buff += read
            else:
                break
        return buff
            
    #Funció per escriptura de comandes AT
    def port_write(self,command):
        while 1:
            try:
                print 'Send',self.name +':',command
                self.netcat.sendall(command + '\n')
                break
            except:
                try:
                    self.netcat = socket.socket(socket.AF_INET, 
                    socket.SOCK_STREAM)
                    self.netcat.settimeout(0.1)     
                    self.netcat.connect((self.ip, self.port))
                    print "Connexio recuperada"
                except:
                    print "Intent de connexio fallit"
                    time.sleep(2)
                    pass
        return
   
    #Funció d'enviament de comandes AT amb comprovació de resposta
    def send(self,command):
        while 1:
            self.port_write(command)
            read = self.port_read()
            x1 = read.find("BUSY CLOSING CONNECTION")
            x2 = read.find("FAILED")
            x3 = read.find("OK")
            if x1!=-1:
                print "Received", self.name + ':', "FAILED"
            elif x2!=-1:
                print "Received",self.name + ':', "BUSY CLOSING CONNECTION"
            elif x3!=-1: 
                print "Received",self.name + ':', "OK"
                break
            elif read == "":
                print "buit"
            else:
                break
            time.sleep(2)
        return read
    
    #Funció per el tancament del socket en cas d'interrupció del programa
    def close(self):
        self.netcat.shutdown(socket.SHUT_WR)
        self.netcat.shutdown(socket.SHUT_RD)
        self.netcat.close()

#Funcions######################################################################

#Funció per separar la imatge en un cert numero de bytes fora de la classe 
def paquets(foto,longitud):
    return [foto[i:i + longitud] for i in range (0,len(foto), longitud)]


#Principal#####################################################################
    
#Definició del mòdem amb la IP, el port de comunicació i el nom (crida de la
#classe netcat)
modem2 = netcat(HOSTNAME1,PORT1,'modem2')

#Obtencio del path on s'ubica el programa 
dir = os.path.dirname(os.path.abspath(__file__))
  
#Definició variables inicials
llistatemps = []
llistatemps_aux = []
llistamissatges = []
llistafotos = [] 
img_correctes = 0
img=0

#Inicialització de la càmera i establiment de la resolució i qualitat
camera = PiCamera()
camera.resolution=(640,480)
qualitat = 10

#Bucle infinit principal 
while 1:
    #Creació de directoris segons els dies
    save_path1 = dir+'/'+str(datetime.date.today())
    
    try:
        os.stat(save_path1)
    except:
        os.mkdir(save_path1)
        llistatemps = []
        llistatemps_aux = []
        llistamissatge = []
        llistafotos = []
        img=0
        img_correctes = 0
                
    #Captacio del temps de inici de captura de imatges
    temps=time.time()
    #Captura de la imatge
    camera.capture(save_path1+'/'+str(temps)+'.jpg',quality=qualitat)
    #Codificacio de la imatge en base64
    with open(save_path1+'/'+str(temps)+'.jpg',"rb") as imageFile:
        imatge = base64.b64encode(imageFile.read())
    
    #Creacio de arxius excel amb informacio relativa al enviament 
    workbook=xlsxwriter.Workbook(save_path1+'/fitxerEstadistic.xlsx')    
    worksheet = workbook.add_worksheet()
    row = 1
    col = 0

    bold = workbook.add_format({'bold': True})
    red = workbook.add_format({'color':'red'})

    worksheet.write(0,0,'Nom Imatge',bold)
    worksheet.write(0,1,'Temps en (s)',bold)
    worksheet.write(0,2,'Estat Imatge',bold)
    worksheet.write(0,3,'Temps mitja en (s)',bold)
    worksheet.write(0,4,'Desviacio estandard',bold)
    worksheet.write(0,5,'Percentatge de fotos enviades',bold)
    worksheet.write(0,6,'Numero de mostres',bold)    
    
    #Obtencio de la mida de la imatge, i creacio de paquets de 1024 bytes
    mida = len(imatge)
    nimatge = paquets(imatge,1024)  
    
    #Generacio d'un fitxer amb possibles errors
    fitxer = os.path.join(save_path1,"MissatgeError.txt")
    
    #Enviament del numero de bytes de la imatge a transmetre
    resposta = modem2.send('AT*SEND,'+str(len(str(mida))+1)+',3,'+
    str(mida)+'#')

    #Comprovacio de la correcte recepcio del numero de bytes 
    t = 0    
    while 1:
        resposta2 = modem2.port_read()
        x=resposta2.find("OK")
        if x!= -1:
            break
        else:
            t = t+1
            if t == 100:
                break;
                
    #Si el modem respon, envia la imatge amb un flag final de exclamacio
    if t != 100:
        i = 0
        while i<len(nimatge):
            if i == len(nimatge)-1:
                resposta3 = modem2.send('AT*SEND,'+str(len(nimatge[i])+1)+
                ',3,'+str(nimatge[i]+"!"))
            else:
                resposta3 = modem2.send('AT*SEND,'+str(len(nimatge[i]))+
                ',3,'+str(nimatge[i]))
            i = i+1 
       
        #Un cop es enviada la imatge s'espera la confirmacio de la recepcio
        #"OK" en cas d'haver sigut correctament enviada, "KO" en cas d'un 
        #enviament erroni o, si el modem receptor es desconecta, "Disconnect"
        t = 0
        aux1=1
        missatge = ''
        while 1:
            resposta = modem2.port_read()
            x=resposta.find("OK")
            x1=resposta.find("KO")
            #Si la resposta es correcta, s'elimina la imatge de la Raspberry
            if x!= -1:
                os.system('rm '+save_path1+'/'+str(temps)+'.jpg')
                img_correctes = img_correctes+1
                missatge = "OK"
                tempsfinal = time.time()-temps
                llistatemps.append(tempsfinal)
                llistatemps_aux.append(tempsfinal)
                break
            #En cas d'incorrecte recepció, s'emmagatzema la imatge, i es
            #guarden els missatges d'error
            else:
                if x1!=-1:
                    try:
                        os.stat(save_path1)
                    except:
                        os.mkdir(save_path1)
                    
                    missatge = "KO" 
                    llistatemps_aux.append("")
                    fh=open(fitxer,"a")
                    fh.write(str(temps)+': '+'El numero de bytes de la imatge'+
                    ' rebuda no coincideix amb la original'+'\n')
                    fh.close
                    break
                #Si no s'obté resposta, es comprova l'estat de comunicació dels
                #mòdems acústics i es finalitza l'enviament, reiniciant el 
                #buffer de transmissio
                else:
                    resposta = modem2.send("AT?S")
                    aux1 =1
                    x = resposta.find("INITIATION LISTEN")
                    x2= resposta.find("1000000 bytes free")
                    #Si es detecta que una perdua de la connexio, degut a un
                    #microtall d'aliementacio o de connexio ethernet, s'envia
                    #el flag "$" per informar al usuari que deixi de llegir i 
                    #esperi la nova rescepcio del numero de bytes
                    if x!=-1:
                        aux1 =0
                        modem2.send('AT*SEND,1,3,$')
                        time.sleep(3)
                        modem2.send("ATZ4")
                    if x2!=-1:
                        t=t+1
                    if t == 50 or aux1 == 0:
                        try:
                            os.stat(save_path1)
                        except:
                            os.mkdir(save_path1)
                            
                        missatge = "Disc." 
                        llistatemps_aux.append("")
                            
                        fh=open(fitxer,"a")
                        fh.write(str(temps)+': '+'El client es va'+
                        ' desconnectar mentres la imatge era enviada'+'\n')
                        fh.close
                            
                        break;
    
    #Si el client no contesta al cap d'un temps "t", es guarda la imatge i es
    #genera un missatge d'error en el fitxer
    else:
        missatge = "OFF" 
        llistatemps_aux.append("")
        try:
            os.stat(save_path1)
        except:
            os.mkdir(save_path1)
            
        fh=open(fitxer,"a")
        fh.write(str(temps)+': '+'El client estava desconnectat'+'\n')
        fh.close
        
    #S'omple l'excel amb les dades
    img=img+1
    llistamissatges.append(missatge)
    llistafotos.append(temps)
    
    if len(llistatemps)==0:
        temps_mitja = ""
        desviacio_estandard = ""
        percentatge = "0"
    else:
        temps_mitja = sum(llistatemps)/len(llistatemps)
        desviacio_estandard = numpy.std(llistatemps)
        percentatge = (img_correctes*100)/img


    worksheet.write(1,3,temps_mitja)
    worksheet.write(1,4,desviacio_estandard)
    worksheet.write(1,5,percentatge)
    worksheet.write(1,6,img)
    
    row=1
        
    for temps in (llistatemps_aux):
        worksheet.write(row,1,temps)
        row+=1
    
    row = 1
    i=0
    
    for missatges in (llistamissatges):
        x1 = missatges.find("OK")
        if x1!=-1:
            worksheet.write(row,2,missatges)
            worksheet.write(row,0,llistafotos[i])
        else:
            worksheet.write(row,2,missatges,red)
            worksheet.write(row,0,llistafotos[i],red)
        row+=1
        i+=1
        
    row = 1
    workbook.close()
    
modem2.close()
