 # -*- coding: utf-8 -*-
"""
Created on Mon Feb 18 23:27:35 2019

@author: Oriol Solé

Programa versió2: Enviament d'imatges configurable (Programa modem transmissor)
"""

#Llibreries####################################################################

#Importacio de les llibreries adients:
import base64   #Llibreria per codificar i descodificar les imatges en base 64
import socket   #Llibreria per fer us dels sockets
import time     #Llibreria per manipular tasques relacionades amb temps
import datetime #Llibreria per obentir dates
import os       #Llibreria per manipular paths
import numpy    #Libreria per executar operacions estaistiques
import xlsxwriter #Llibreria per operar amb fitxers excel 
from picamera import PiCamera#Llibreria per controlar la camera de la Raspberry

#Definicio Modems##############################################################

#Definicio de les IPs i dels ports de comunicació dels modems acustics físics 
#i virtuals transmissors
HOSTNAME1EM = '10.42.57.2'
HOSTNAME1 = '192.168.1.196'
PORT1 = 9200

#Classe funcions sockets#######################################################

#Creacio de la classe netcat per ubicar les funcions relatives a la manipulacio
#dels sockets        
class netcat(object):
    def __init__ (self,hostname,port,modem_name):
        #Temps d'espera per assegurar la connectivitat ethernet de la Raspberry
        #al iniciar-se 
        time.sleep(10)
        #Establiment de la ip, el port de comunicació i el nom del mòdem
        self.ip = hostname
        self.port = port
        self.name = modem_name
        #Creació del socket i connectivitat
        try:
            self.netcat = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            self.netcat.settimeout(0.1)     
            self.netcat.connect((self.ip, self.port))
        except:
            print "No s'ha pogut establir connexio"
        
       #Configuracions inicials:
        print 'Activacio permissos'
        self.send('AT@CTRL')
        print 'Desactivacio notificacions extesses'
        self.send('AT@ZX0')
        print 'Desactivacio notificacions de posicionament'
        self.send('AT@ZU0')
        print 'Reiniciament del buffer de transmissio:'
        self.send('ATZ4')
        print 'Configuracio del numero de bytes del buffer de transmissio:'
        self.send('AT@ZL1000000')
        print 'Guardar en memoria les configuracions del modem'
        self.send('AT&W') 
        
    #Funcio de lectura de dades     
    def port_read(self):
        buff = ''
        aux = 0
        read = ''
        while 1:
            #Bloc d'excepcio, en cas de no poder establir connectivitat, i que
            #no es tracti d'un error de timeout, es torna a intentar restablir
            #la connexio perduda
            try:
                read = self.netcat.recv(1)
            except socket.error as error:
                if str(error)!= "timed out":
                    try:
                        self.netcat = socket.socket(socket.AF_INET,
                        socket.SOCK_STREAM)
                        self.netcat.settimeout(0.1)     
                        self.netcat.connect((self.ip, self.port))
                        print "Connexio recuperada"
                    except:
                        print "Intent de connexio fallit"
                        time.sleep(2)
                break
            #Si es troba el flag de cancelacio de l'enviament d'imatges "@" es 
            #retorna la variable aux per tal de interrompre l'enviament
            x=read.find("@")
            if x!=-1:
                aux=1
            if read:
                buff += read
            else:
                break
        return buff,aux
    
    #Funció per escriptura de comandes AT
    def port_write(self,command):
        while 1:
            try:
                print 'Send',self.name +':',command
                self.netcat.sendall(command + '\n')
                break
            except:
                try:
                    self.netcat = socket.socket(socket.AF_INET, 
                    socket.SOCK_STREAM)
                    self.netcat.settimeout(0.1)     
                    self.netcat.connect((self.ip, self.port))
                    print "Connexio recuperada"
                except:
                    print "Intent de connexio fallit"
                    time.sleep(2)
                    pass
        return
        
    #Funció d'enviament de comandes AT amb comprovació de resposta    
    def send(self,command):
        while 1:
            self.port_write(command)
            read = self.port_read()
            x1 = read[0].find("BUSY CLOSING CONNECTION")
            x2 = read[0].find("FAILED")
            x3 = read[0].find("OK")
            if x1!=-1:
                print "Received", self.name + ':', "FAILED"
            elif x2!=-1:
                print "Received",self.name + ':', "BUSY CLOSING CONNECTION"
            elif x3!=-1: 
                print "Received",self.name + ':', "OK"
                break
            elif read[0] == "":
                print 'buit' 
                time.sleep(2)
            else:
                break
            time.sleep(2)
        return read[0],read[1]
    
    #Funció per el tancament del socket en cas d'interrupció del programa       
    def close(self):
        self.netcat.shutdown(socket.SHUT_WR)
        self.netcat.shutdown(socket.SHUT_RD)
        self.netcat.close()
        
#Funcions######################################################################

#Funció per separar la imatge en un cert numero de bytes fora de la classe 
def lista(foto,longitud):
    return [foto[i:i + longitud] for i in range (0,len(foto), longitud)]


#Principal#####################################################################

#Definició del mòdem amb la IP, el port de comunicació i el nom (crida de la
#classe netcat)    
modem2 = netcat(HOSTNAME1,PORT1,'modem2')

#Obtencio del path on s'ubica el programa 
dir = os.path.dirname(os.path.abspath(__file__))

#Definició variables inicials
llistatemps = []
llistatemps_aux = []
llistamissatge = []
llistafotos = []
NumeroSerie = 0
#Inicialització de la càmera
camera = PiCamera()
#Creacio del directori global
save_pathGlobal = dir+'/Results'  
try:                           
    os.stat(save_pathGlobal)   
except:
    os.mkdir(save_pathGlobal)
#Creacio del directori amb les series on aniran les imatges
save_pathGlobalFotos = save_pathGlobal+'/Series'
try:                           
    os.stat(save_pathGlobalFotos)   
except:
    os.mkdir(save_pathGlobalFotos)
    
#Bucle infinit Principal
while 1:
    #Declaracio i inicialitzacio variables amb espera del numero de fotos a 
    #capturar, la resolucio i la qualitat
    NumeroSerie = NumeroSerie+1
    aux4 = 0
    img_correctes = 0
    img=0
    llistatemps=[]
    llistamissatge=[]
    llistafotos=[]
    llistatemps_aux=[]
    while 1:
        info = modem2.port_read()
        x= info[0].find("!")
        if x!=-1:
             info = info[0].replace("!","")
             break
    
    #Creacio de les series amb amb les dates
    save_pathSerien=save_pathGlobalFotos+'/'+str((str(NumeroSerie)).zfill(4))\
    +'_'+str(datetime.date.today())
    #Es guarda la informació obtinguda en diferents variables
    info = info.split(",")
    Nfotos =int(info[9])
    Resolucio1= int(info[10])
    Resolucio2= int(info[11])
    Qualitat = int(info[12])
    
    #S'estableix la resolucio i es crea bucle while per el numero de fotos a 
    #enviar
    while img<Nfotos:
        camera.resolution=(Resolucio1,Resolucio2)
        try:
            os.stat(save_pathSerien)
        except:
            os.mkdir(save_pathSerien)
        
        #Es captura el temps per calcular la duracio de l'enviament
        temps=time.time()
        color = 0
        #Generacio de fitxers d'errors i d'imatges
        fichero = os.path.join(save_pathSerien,"MissatgeError.txt")
        fichero1= os.path.join(save_pathSerien,str(temps)+".jpg")
        #Captura de la imatge
        camera.capture(save_pathSerien+'/'+str(temps)+'.jpg',quality= Qualitat)   
        #Codificacio de la imatge en base64
        with open(save_pathSerien+'/'+str(temps)+'.jpg',"rb") as imageFile:
            nuevo = base64.b64encode(imageFile.read())
          
        #Obtencio de la mida de la imatge, i creacio de paquets de 1024 bytes 
        valor = lista(nuevo,1024)   
        size = len(nuevo)
        
        #Enviament del numero de bytes de la imatge a transmetre
        modem2.send('AT*SEND,'+str(len(str(size))+1)+',3,'+str(size)+'#')
                    
        #Comprovacio de la correcte recepcio del numero de bytes 
        t = 0
        while 1:
            if aux4==1:
                t=50
                break
            resposta2 = modem2.port_read()
            x=resposta2[0].find("OK")
            if resposta2[1]==1:
                aux4=1
                t=50
                break
            if x!= -1:     
                break
            else:
                t = t+1
                if t == 100:
                    break;
                    
        #Si el modem respon, envia la imatge amb un flag final de exclamacio  
        if t != 100:
            i = 0
        #S'envia la imatge i es comprova si l'usuari ha demanat la cancelacio
        #de les mateixes amb el flag "@"
            while i<len(valor):
                if aux4 ==1:
                    break
                resposta = modem2.port_read()
                if resposta[1] == 1:
                    aux4=1
                    break
                if i == len(valor)-1:
                    resposta3 = modem2.send('AT*SEND,'+str(len(valor[i])+1)+
                    ',3,'+str(valor[i]+"!"))
                    if resposta3[1]==1:
                        aux4=1
                        break      
                else:
                    resposta3 = modem2.send('AT*SEND,'+str(len(valor[i]))+
                    ',3,'+str(valor[i]))
                    if resposta3[1]==1:
                        aux4=1
                        break
                i = i+1 
              
            t = 0
            #Un cop es enviada la imatge s'espera la confirmacio de la recepcio
            #"OK" en cas d'haver sigut correctament enviada, "KO" en cas d'un 
            #enviament erroni si el modem receptor es desconecta,"Disconnect"
            #o si s'ha cancelat l'enviament d'imatges "Cancel"
            while 1:
                if aux4 == 1:
                    missatge = "Cancel"
                    llistatemps_aux.append("")
                    color = 1
                    try:
                        os.stat(save_pathSerien)
                    except:
                        os.mkdir(save_pathSerien)
                    
                    fh=open(fichero,"a")
                    fh.write(str(temps)+': '+"La imatge no s'ha enviat perque"+
                    "el client ha cancelat l'enviament de les imatges"+'\n')
                    fh.close
                                            
                    fh =open(fichero1,"wb")
                    fh.write(nuevo.decode('base64'))
                    fh.close
                    break
                #Si no es detecta la cancelacio es comproven les respostes
                resposta2 = modem2.port_read()
                x=resposta2[0].find("OK")
                x1=resposta2[0].find("KO")
                #Si la resposta es correcta, s'elimina la imatge 
                if x!= -1:
                    img_correctes = img_correctes+1
                    missatge = "OK"
                    os.system('rm '+save_pathSerien+'/'+str(temps)+'.jpg')
                    tempsfinal = time.time()-temps
                    llistatemps.append(tempsfinal)
                    llistatemps_aux.append(tempsfinal)
                    break
                else:
                    #En cas d'incorrecte recepció, s'emmagatzema la imatge, i
                    #es guarden els missatges d'error
                    if x1!=-1:
                        color=1
                        missatge = "KO"  
                        llistatemps_aux.append("")
                        try:
                            os.stat(save_pathSerien)
                        except:
                            os.mkdir(save_pathSerien)
                    
                        fh=open(fichero,"a")
                        fh.write(str(temps)+': '+'El numero de bytes de la'+
                        'imatge rebuda no coincideix amb la original'+"\n")
                        fh.close
         
                        fh =open(fichero1,"wb")
                        fh.write(nuevo.decode('base64'))
                        fh.close
                        
                        break
                     #Si no s'obté resposta, es comprova l'estat de comunicació
                     #dels mòdems acústics i es finalitza l'enviament, 
                     #reiniciant el buffer de transmissio
                    else:
                        if resposta2[1]==1:
                            aux4=1
                            
                        resposta = modem2.send("AT?S")
                        aux1 =1
                        print resposta
                        x = resposta[0].find("INITIATION LISTEN")
                        x2= resposta[0].find("1000000 bytes free")
                        if x!=-1:
                            aux1 =0
                            modem2.send("ATZ4")
                        if x2!=-1:
                            t=t+1
                        if resposta[1] == 1:
                            aux4 = 1
                        if t == 50 or aux1 == 0:
                            missatge = "Disc." 
                            llistatemps_aux.append("")
                            color = 1
                            try:
                                os.stat(save_pathSerien)
                            except:
                                os.mkdir(save_pathSerien)
                            
                            fh=open(fichero,"a")
                            fh.write(str(temps)+': '+'El client es va'+
                            'desconnectar mentres la imatge era enviada'+'\n')
                            fh.close
                                    
                            fh =open(fichero1,"wb")
                            fh.write(nuevo.decode('base64'))
                            fh.close
                            break;
        
        #Si el client no contesta al cap d'un temps "t", es guarda la imatge i
        #es genera un missatge d'error en el fitxer
        else:
            missatge = "OFF"   
            llistatemps_aux.append("")
            color=1
            try:
                os.stat(save_pathSerien)
            except:
                os.mkdir(save_pathSerien)
                
                    
            fh=open(fichero,"a")
            fh.write(str(temps)+': '+'El client estava desconnectat'+'\n')
            fh.close
                                    
            fh =open(fichero1,"wb")
            fh.write(nuevo.decode('base64'))
            fh.close
        
        img=img+1
        llistamissatge.append(missatge)
        llistafotos.append(temps)
        
        if (aux4==1):
            print 'Reset Buffer transmission:'
            modem2.send('ATZ4')
            break
    
    #En cas d'haver completat el numero de fotos previstes d'enviar:
    if len(llistatemps) == 0:
        temps_mitja = ""
        desviacio_estandard = ""
        percentatge= "0"
    else:
    
        temps_mitja = sum(llistatemps)/len(llistatemps)
        desviacio_estandard = numpy.std(llistatemps)
        percentatge = (img_correctes*100)/img
        
    try:
        os.stat(save_pathSerien)
    except:
        os.mkdir(save_pathSerien)
        
    #Creacio de arxius excel amb informacio relativa al enviament de imatges
    workbook=xlsxwriter.Workbook(save_pathSerien+'/fitxerEstadistic.xlsx')
    worksheet = workbook.add_worksheet()
    row = 1
    i=0
    bold = workbook.add_format({'bold': True})
    red = workbook.add_format({'color':'red'})
    
    worksheet.write(0,0,'Nom Imatge',bold)
    worksheet.write(0,1,'Temps en (s)',bold)
    worksheet.write(0,2,'Estat Imatge',bold)
    worksheet.write(0,3,'Temps mitja en (s)',bold)
    worksheet.write(1,3,temps_mitja)
    worksheet.write(0,4,'Desviacio estandard',bold)
    worksheet.write(1,4,desviacio_estandard)
    worksheet.write(0,5,'Percentaje de fotos enviades',bold)
    worksheet.write(1,5,percentatge)
    worksheet.write(0,6,'Numero de mostres',bold)
    worksheet.write(1,6,img)
    
    row = 1
    
    for temps in (llistatemps_aux):
        worksheet.write(row,1,temps)
        row+=1
    
    row = 1
    
    for missatges in (llistamissatge):
        x1 = missatges.find("OK")
        if x1!=-1:
            worksheet.write(row,2,missatges)
            worksheet.write(row,0,llistafotos[i])
        else:
            worksheet.write(row,2,missatges,red)
            worksheet.write(row,0,llistafotos[i],red)
        row+=1
        i+=1
    workbook.close()
    
    #S'envia el flag per avisar al modem receptor de que s'ha finalitzat amb
    #l'enviament de les imatges previstes
    modem2.send('AT*SEND,1,3,$')
            
modem2.close()
